<?php

namespace Drupal\simply_signups\Controller;

use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Component\Datetime\TimeInterface;
use Symfony\Component\HttpFoundation\Response;
use Drupal\simply_signups\Utility\SimplySignupsUtility;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

class SimplySignupsController extends ControllerBase {

  /**
   * Display the markup.
   *
   * @return array
   */
  public function dashboardPage() {
    $config = \Drupal::config('simply_signups.config');
    $bundles = $config->get('bundles');
    $rows = [];
    $header = [
      'title' => [
        'data' => t('title'),
        'field' => 'title',
        'specifier' => 'title',
      ],
      'start_date' => [
        'data' => t('start date'),
      ],
      'end_date' => [
        'data' => t('end date'),
      ],
      'status' => [
        'data' => t('published'),
        'field' => 'status',
        'specifier' => 'status',
      ],
      'attending' => [
        'data' => t('attending'),
      ],
      'checkedin' => [
        'data' => t('checked in'),
      ],
      'operations' => [
        'data' => t('operations'),
      ],
    ];
    if (!empty($bundles)) {
      $nids = \Drupal::entityQuery('node')
        ->tableSort($header)
        ->pager(50)
        ->condition('type', $bundles, 'IN')
        ->condition('status', 1, '=')
        ->execute();
      foreach ($nids as $nid) {
        $node = \Drupal\node\Entity\Node::load($nid);
        $id = $node->id();
        $title = $node->getTitle();
        $type = $node->bundle();
        $status = ($node->isPublished()) ? 'Yes' : 'No';

        /* generate node link */
        $nodeUrl = Url::fromRoute('entity.node.canonical', ['node' => $nid]);
        $nodeLink = Link::fromTextAndUrl($title, $nodeUrl)->toString();

        /*
         * generate node edit link
         * include a destination url to redirect the user back to the dashboard after editing a node
         */
        $destinationUrl = Url::fromRoute('simply_signups.dashboard');
        $submissions = SimplySignupsUtility::get_number_of_signups($nid);
        $attending = SimplySignupsUtility::get_number_of_attending($nid);
        $checkedin = SimplySignupsUtility::get_number_of_checkedins_attending($nid);
        $startDate = (SimplySignupsUtility::get_start_date($nid) != FALSE) ? date('m/d/Y - h:i a', SimplySignupsUtility::get_start_date($nid)) : '-';
        $endDate = (SimplySignupsUtility::get_end_date($nid) != FALSE) ? date('m/d/Y - h:i a', SimplySignupsUtility::get_end_date($nid)) : '-';

        $links['edit'] = [
          'title' => $this->t('edit'),
          'url' => Url::fromRoute('entity.node.edit_form', ['node' => $nid, 'destination' => $destinationUrl->toString()]),
        ];
        $links['configure'] = [
          'title' => $this->t('configure'),
          'url' => Url::fromRoute('simply_signups.nodes.settings', ['node' => $nid, 'destination' => $destinationUrl->toString()]),
        ];
        if ($submissions > 0) {
          $links['rsvps'] = [
            'title' => $this->t('signups'),
            'url' => Url::fromRoute('simply_signups.nodes', ['node' => $nid]),
          ];
          $links['download_rsvps'] = [
            'title' => $this->t('download'),
            'url' => Url::fromRoute('simply_signups.nodes.csv', ['node' => $nid]),
          ];
        }
        $rows[$id] = [
          'data' => [
            'title' => $nodeLink,
            'start_date' => $startDate,
            'end_date' => $endDate,
            'status' => $status,
            'attending' => $attending,
            'checkedin' => $checkedin,
            'operations' => [
              'data' => [
                '#type' => 'dropbutton',
                '#links' => $links,
              ],
            ],
          ],
        ];
        unset($links['download_rsvps']);
      }
    }
    $url = Url::fromRoute('simply_signups.config');
    $link = Link::fromTextAndUrl('configure', $url)->toString();
    $build['table'] = [
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => t('There are 0 items currently found. Don\'t forget to @link simply signups.', ['@link' => $link]),
      '#attributes' => [
        'data-striping' => 1,
      ],
    ];
    $build['pager'] = [
      '#type' => 'pager',
    ];
    return $build;
  }

  public function dashboardUnpublishedPage() {
    $config = \Drupal::config('simply_signups.config');
    $bundles = $config->get('bundles');
    $rows = [];
    $header = [
      'title' => [
        'data' => t('title'),
        'field' => 'title',
        'specifier' => 'title',
      ],
      'start_date' => [
        'data' => t('start date'),
      ],
      'end_date' => [
        'data' => t('end date'),
      ],
      'status' => [
        'data' => t('published'),
        'field' => 'status',
        'specifier' => 'status',
      ],
      'attending' => [
        'data' => t('attending'),
      ],
      'checkedin' => [
        'data' => t('checked in'),
      ],
      'operations' => [
        'data' => t('operations'),
      ],
    ];
    if (!empty($bundles)) {
      $nids = \Drupal::entityQuery('node')
        ->tableSort($header)
        ->pager(50)
        ->condition('type', $bundles, 'IN')
        ->condition('status', 0, '=')
        ->execute();
      foreach ($nids as $nid) {
        $node = \Drupal\node\Entity\Node::load($nid);
        $id = $node->id();
        $title = $node->getTitle();
        $type = $node->bundle();
        $status = ($node->isPublished()) ? 'Yes' : 'No';

        /* generate node link */
        $nodeUrl = Url::fromRoute('entity.node.canonical', ['node' => $nid]);
        $nodeLink = Link::fromTextAndUrl($title, $nodeUrl)->toString();

        /*
         * generate node edit link
         * include a destination url to redirect the user back to the dashboard after editing a node
         */
        $destinationUrl = Url::fromRoute('simply_signups.dashboard');
        $submissions = SimplySignupsUtility::get_number_of_signups($nid);
        $attending = SimplySignupsUtility::get_number_of_attending($nid);
        $checkedin = SimplySignupsUtility::get_number_of_checkedins_attending($nid);
        $startDate = (SimplySignupsUtility::get_start_date($nid) != FALSE) ? date('m/d/Y - h:i a', SimplySignupsUtility::get_start_date($nid)) : '-';
        $endDate = (SimplySignupsUtility::get_end_date($nid) != FALSE) ? date('m/d/Y - h:i a', SimplySignupsUtility::get_end_date($nid)) : '-';

        $links['edit'] = [
          'title' => $this->t('edit'),
          'url' => Url::fromRoute('entity.node.edit_form', ['node' => $nid, 'destination' => $destinationUrl->toString()]),
        ];
        $links['configure'] = [
          'title' => $this->t('configure'),
          'url' => Url::fromRoute('simply_signups.nodes.settings', ['node' => $nid, 'destination' => $destinationUrl->toString()]),
        ];
        if ($submissions > 0) {
          $links['rsvps'] = [
            'title' => $this->t('signups'),
            'url' => Url::fromRoute('simply_signups.nodes', ['node' => $nid]),
          ];
          $links['download_rsvps'] = [
            'title' => $this->t('download'),
            'url' => Url::fromRoute('simply_signups.nodes.csv', ['node' => $nid]),
          ];
        }
        $rows[$id] = [
          'data' => [
            'title' => $nodeLink,
            'start_date' => $startDate,
            'end_date' => $endDate,
            'status' => $status,
            'attending' => $attending,
            'checkedin' => $checkedin,
            'operations' => [
              'data' => [
                '#type' => 'dropbutton',
                '#links' => $links,
              ],
            ],
          ],
        ];
        unset($links['download_rsvps']);
      }
    }
    $url = Url::fromRoute('simply_signups.config');
    $link = Link::fromTextAndUrl('configure', $url)->toString();
    $build['table'] = [
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => t('There are 0 items currently found. Don\'t forget to @link simply signups.', ['@link' => $link]),
      '#attributes' => [
        'data-striping' => 1,
      ],
    ];
    $build['pager'] = [
      '#type' => 'pager',
    ];
    return $build;
  }

  public function templatesEditForm() {
    $path = \Drupal::service('path.current')->getPath();
    $currentPath = ltrim($path, '/');
    $arg = explode('/', $currentPath);
    $request = 'invalid';
    $operation = 'invalid';
    if ($arg[3] == 'templates') {
      $request = $arg[3];
      if ($arg[5] == 'add') {
        $operation = $arg[5];
      }
      if ($arg[5] == 'edit') {
        $operation = $arg[5];
        $fid = $arg[6];
      } 
      $tid = $arg[4];
    }
    if ($arg[0] == 'node') {
      $request = $arg[0];
      if ($arg[4] == 'add') {
        $operation = $arg[4];
      }
      if ($arg[4] == 'edit') {
        $operation = $arg[4];
        $fid = $arg[5];
      } 
    }
    $db = \Drupal::database();
    $query = $db->select('simply_signups_templates', 'p');
    $query->fields('p');
    $query->condition('id', $tid, '=');
    $count = $query->countQuery()->execute()->fetchField();
    if ($count == 0) {
      throw new NotFoundHttpException();
    }
    $query = $db->select('simply_signups_templates_fields', 'p');
    $query->fields('p');
    $query->condition('id', $fid, '=');
    $count = $query->countQuery()->execute()->fetchField();
    if ($count == 0) {
      throw new NotFoundHttpException();
    }
    $results = $query->execute()->fetchAll();
    foreach ($results as $row) {
      $field = unserialize($row->field);
    }
    $type = ucfirst($field['#type']);
    $form = \Drupal::formBuilder()->getForm('Drupal\simply_signups\Form\Field\SimplySignupsFields'.$type.'Form');
    return $form;
  }
  public function nodesEditForm() {
    $path = \Drupal::service('path.current')->getPath();
    $currentPath = ltrim($path, '/');
    $arg = explode('/', $currentPath);
    $request = 'invalid';
    $operation = 'invalid';
    if ($arg[0] == 'node') {
      $request = $arg[0];
      if (($arg[5] == 'add') OR ($arg[5] == 'edit')) {
        $request = $arg[5];
      }
    }
    if ($arg[3] == 'templates') {
      $request = $arg[3];
      if (($arg[4] == 'add') OR ($arg[4] == 'edit')) {
        $operation = $arg[4];
      }
    }
    $nid = $arg[1];
    $node = Node::load($nid);
    $isValidNode = (isset($node)) ? TRUE : FALSE;
    if (!$isValidNode) {
      throw new NotFoundHttpException();
    }
    $id = $node->id();
    $fid = $arg[5];
    $db = \Drupal::database();
    $query = $db->select('simply_signups_fields', 'p');
    $query->fields('p');
    $query->condition('id', $fid, '=');
    $count = $query->countQuery()->execute()->fetchField();
    if ($count == 0) {
      throw new NotFoundHttpException();
    }
    $results = $query->execute()->fetchAll();
    foreach ($results as $row) {
      $field = unserialize($row->field);
    }
    $type = ucfirst($field['#type']);
    $form = \Drupal::formBuilder()->getForm('Drupal\simply_signups\Form\Field\SimplySignupsFields'.$type.'Form');
    return $form;
  }
  public function downloadSignups() {
    $path = \Drupal::service('path.current')->getPath();
    $currentPath = ltrim($path, '/');
    $arg = explode('/', $currentPath);
    $id = $arg[1];
    $node = Node::load($id);
    $nid = $node->id();
    $output = [];
    $isValidNode = (is_numeric($nid)) ? TRUE : FALSE;
    if (!$isValidNode) {
      throw new NotFoundHttpException();
    }
    $title = $node->getTitle();
    $db = \Drupal::database();
    $query = $db->select('simply_signups_fields', 'p');
    $query->fields('p');
    $query->orderBy('weight');
    $query->condition('nid', $nid, '=');
    $count = $query->countQuery()->execute()->fetchField();

    $options = ['absolute' => TRUE];
    $nodeUrl = Url::fromRoute('entity.node.canonical', ['node' => $nid], $options);
    $nodeLink = Link::fromTextAndUrl('Return to node', $nodeUrl)->toString();

    if ($count == 0) {
      $url = Url::fromRoute('simply_signups.nodes.fields', ['node' => $nid], $options);
      $link = Link::fromTextAndUrl('Manage fields', $url)->toString();
      $output = ['#markup' => t('There are currently 0 <em>signups fields</em> for this event.<br />'. $nodeLink. '<br />'. $link)];
    }
    else {
      $results = $query->execute()->fetchAll();
      $rawHeadings = [];
      foreach ($results as $result) {
        $rawHeadings[] = $result->name;
      }
      $database = \Drupal::database();
      $q = $database->select('simply_signups_settings', 'p');
      $q->fields('p');
      $q->condition('nid', $nid, '=');
      $events = $q->execute()->fetchAll();
      foreach ($events as $event) {
        $event_name = $event->name;
        $event_location = $event->location;
        $event_address_line1 = $event->address_line1;
        $event_address_line2 = $event->address_line2;
        $event_locality = $event->locality;
        $event_administrative_area = $event->administrative_area;
        $event_postal_code = $event->postal_code;
        $event_country_code = $event->country_code;
        $event_start_date_raw = $event->event_start_date;
        $event_end_date_raw = $event->event_end_date;
      }
      $event_address = $event_address_line1. ' - ';
      $event_address .= (!empty($event_address_line2)) ? $event_address_line2. ' - ' : '';
      $event_address .= $event_locality. ' - ';
      $event_address .= $event_administrative_area. ' - ';
      $event_address .= $event_postal_code;
      $event_start_date = format_date($event_start_date_raw, 'custom', 'm/d/Y');
      $event_end_date = format_date($event_end_date_raw, 'custom', 'm/d/Y');
      $event_start_time = format_date($event_start_date_raw, 'custom', 'h:i a');
      $event_end_time = format_date($event_end_date_raw, 'custom', 'h:i a');
      $headings = implode(",", $rawHeadings);
      $query = $db->select('simply_signups_data', 'p');
      $query->fields('p');
      $query->condition('nid', $nid, '=');
      $count = $query->countQuery()->execute()->fetchField();
      if ($count == 0) {
        $options = ['absolute' => TRUE];
        $url = Url::fromRoute('entity.node.canonical', ['node' => $nid], $options);
        $nodeLink = Link::fromTextAndUrl('Return to node', $url)->toString();
        $output = ['#markup' => t('There are currently 0 <em>signups</em> for this event.<br />'. $nodeLink)];
      }
      else {
        $results = $query->execute()->fetchAll();
        $allData = [];
        $allData[] = 'Event Name, Event Date, Event Time, Event Location, Event Address, Event Address 2, Event City, Event State, Event ZIP Code, Event URL, '. $headings. ', Check-In, Submitted';
        foreach ($results as $row) {
          $rawSignupData = unserialize($row->fields);
          $signupData = [];
          foreach ($rawSignupData as $field) {
            if ($field['type'] == "tel") {
              $formattedTelephone = SimplySignupsUtility::format_telephone($field['value'], 3);
              $field['value'] = $formattedTelephone;
            }
            $signupData[] = $field['value'];
          }
          $status = ($row->status == 1) ? 'Yes' : 'No';
          $submitted = format_date($row->created, 'custom', 'm/d/Y - h:i a');
          $updated = format_date($row->updated, 'custom', 'm/d/Y - h:i a');
          $allData[] = $event_name. ', '. $event_start_date. ', '. $event_start_time. ', '. $event_location. ', '. $event_address_line1. ', '. $event_address_line2. ', '. $event_locality. ', '. $event_administrative_area. ', '. $event_postal_code. ', '. $nodeUrl->toString(). ', '. implode(",", $signupData). ', '. $status. ', '. $submitted;
        }
        $completeFileData = implode("\n", $allData);
        $requestTime = \Drupal::time()->getCurrentTime();
        $title = $title.'_'.$requestTime;
        $csvFilename = $title.'.csv';
        $filename = $csvFilename;
        $output = $completeFileData;
        $response = new Response();
        $response->headers->set('Content-type', 'text/csv');
        $response->headers->set('Content-Disposition', 'attachment; filename="' . $filename . '"');
        $response->setContent(render($output));
        $response->send();
        exit();
      }
    }
  }
}