<?php

namespace Drupal\simply_signups\Form;

use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\simply_signups\Utility\SimplySignupsUtility;

/**
 * Implements a signup form.
 */
class SimplySignupsListForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'simply_signups_list_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $path = \Drupal::service('path.current')->getPath();
    $currentPath = ltrim($path, '/');
    $arg = explode('/', $currentPath);
    $nid = $arg[1];
    $node = Node::load($nid);
    $id = $node->id();
    $numberOfCheckedInAttending = SimplySignupsUtility::get_number_of_checkedins_attending($id);
    $form['#attached']['library'][] = 'simply_signups/styles';
    $form['#attributes'] = [
      'class' => ['simply-signups-list-form', 'simply-signups-form'],
    ];
    $db = \Drupal::database();
    $query = $db->select('simply_signups_data', 'p');
    $query->fields('p');
    $query->condition('nid', $id, '=');
    $signupCount = $query->countQuery()->execute()->fetchField();
    $attendingCount = 0;
    $attendedCount = 0;
    $statusCount = 0;
    $results = $query->execute()->fetchAll();
    $header = [
      'signup_data' => t('Signup'),
      'number_attending' => t('# Attending'),
      'status' => t('Status'),
      'updated' => t('Updated'),
      'operations' => t('Operations'),
    ];
    $output = array();
    $attendingCount = 0;
    $checkedInCount = 0;
    foreach ($results as $row) {
      $links['status'] = [
        'title' => $this->t('status'),
        'url' => Url::fromRoute('simply_signups.nodes.single', ['node' => $id, 'sid' => $row->id]),
        'attributes' => ['id' => 'signup-list-view-modal', 'class' => ['use-ajax'], 'data-dialog-type' => 'modal', 'data-dialog-options' => '{"width":"100%"}'],
      ];
      $links['edit'] = [
        'title' => $this->t('edit'),
        'url' => Url::fromRoute('simply_signups.nodes.single.edit', ['node' => $id, 'sid' => $row->id]),
      ];
      $links['remove'] = [
        'title' => $this->t('remove'),
        'url' => Url::fromRoute('simply_signups.nodes.single.remove', ['node' => $id, 'sid' => $row->id]),
      ];
      $updatedDate = format_date($row->updated, 'custom', 'm/d/Y - h:i a');
      $attendingCount = ($attendingCount+$row->attending);
      $checkedInCount = ($row->status == 1) ? ($checkedInCount+1) : ($checkedInCount+0);
      $rawSignupData = unserialize($row->fields);
      $signupData = '';
      $x = 0;
      foreach ($rawSignupData as $field) {
        if ($x < 3) {
          if ($field['type'] == "tel") {
            $formattedTelephone = SimplySignupsUtility::format_telephone($field['value'], 3);
            $field['value'] = $formattedTelephone;
          }
          $signupData .= $field['title']. ": ". $field['value']. "<br />";
        }
        $x++;
      }
      $output[($row->id+1)] = [
        'signup_data' => t($signupData),
        'number_attending' => $row->attending,
        'status' => ($row->status == 1) ? t('<div class="btn btn-xs btn-success">Checked-In</div>') : '-',
        'updated' => $updatedDate,
        'operations' => [
          'data' => [
            '#type' => 'dropbutton',
            '#links' => $links,
          ],
        ],
      ];
    }
    $form['total_attending'] = [
      '#markup' => $this->t('<div class="simply-signups-total-attending simply-signups-action-info">Attending: <strong>'. $attendingCount. '</strong></div>'),
    ];
    $form['total_signups'] = [
      '#markup' => $this->t('<div class="simply-signups-total-signups simply-signups-action-info">Submissions: <strong>'. $signupCount. '</strong></div>'),
    ];
    $form['total_checked_in_attending'] = [
      '#markup' => $this->t('<div class="simply-signups-total-attended simply-signups-action-info">Attending (Checked-In): <strong>'. $numberOfCheckedInAttending. '</strong></div>'),
    ];
    $form['total_checked_in'] = [
      '#markup' => $this->t('<div class="simply-signups-total-attended simply-signups-action-info">Submissions (Checked-In): <strong>'. $checkedInCount. '</strong></div>'),
    ];
    if ($signupCount > 0) {
      $options = ['absolute' => TRUE];
      $url = Url::fromRoute('simply_signups.nodes.csv', ['node' => $nid], $options);
      $link = Link::fromTextAndUrl('Download signups', $url)->toString();
      $form['download_csv'] = [
        '#markup' => $this->t('<div class="simply-signups-download-csv simply-signups-action-info">'. $link. '</div>'),
      ];
    }
    $form['table'] = [
      '#type' => 'tableselect',
      '#header' => $header,
      '#options' => $output,
      '#empty' => t('No signups found.'),
    ];
    $form['actions'] = [
      '#type' => 'actions',
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Remove selected items'),
      '#attributes' => [
        'class' => [
          'button--danger',
          'btn-link',
        ],
      ],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $rows = $values['table'];
    $selected = array_filter($rows);
    if (empty($selected)) {
      $form_state->setErrorByName('table', $this->t('Must select at least one item.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $rows = $values['table'];
    $rows = array_filter($rows);
    foreach ($rows as $row) {
      $item = ($row - 1);
      $db = \Drupal::database();
      $query = $db->delete('simply_signups_data');
      $query->condition('id', $item, '=');
      $query->execute();
    }
    drupal_set_message($this->t('Successfully removed selected signup(s).'));
  }
}