<?php

namespace Drupal\simply_signups\Form;

use Drupal\Core\Url;
use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Datetime\TimeInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

use Drupal\signup\Utility\SignupUtility;

/**
 * Implements a signup form.
 */
class SimplySignupsNodesApplyTemplateForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'simply_signups_nodes_apply_template_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $path = \Drupal::service('path.current')->getPath();
    $currentPath = ltrim($path, '/');
    $arg = explode('/', $currentPath);
    $nid = $arg[1];
    $node = Node::load($nid);
    $isValidNode = (isset($node)) ? TRUE : FALSE;
    if (!$isValidNode) {
      throw new NotFoundHttpException();
    }
    $id = $node->id();
    $nid = $id;
    $db = \Drupal::database();
    $query = $db->select('simply_signups_templates', 'p');
    $query->fields('p');
    $count = $query->countQuery()->execute()->fetchField();
    $form['#attached']['library'][] = 'simply_signups/styles';
    $form['#attributes'] = [
      'class' => ['simply-signups-nodes-apply-template-form', 'simply-signups-form'],
    ];
    $form['nid'] = [
      '#type' => 'hidden',
      '#value' => $id,
    ];
    $form['count'] = [
      '#type' => 'hidden',
      '#value' => $count,
    ];
    $results = $query->execute()->fetchAll();
    foreach ($results as $row) {
      $items[$row->id] = $row->title;
    }
    if ($count > 0) {
      $form['templates'] = [
        '#type' => 'select',
        '#title' => $this->t('Choose a template to apply'),
        '#options' => $items,
      ];
    }
    if ($count == 0) {
      $links['view'] = Url::fromRoute('simply_signups.templates');
      $form['not_found_title'] = [
        '#markup' => $this->t('<h3>No templates found</h3>'),
      ];
      $form['not_found_text'] = [
        '#markup' => $this->t('<p>Could not find any templates. If you wish to create one, please visit here to <a href="' .$links['view']->toString(). '">add a template</a></p>'),
      ];
    }
    $form['actions'] = [
      '#type' => 'actions',
    ];
    if ($count > 0) {
      $form['actions']['submit'] = [
        '#type' => 'submit',
        '#value' => $this->t('Apply template'),
      ];
    }
    $form['actions']['cancel'] = [
      '#type' => 'submit',
      '#value'  => 'Cancel',
      '#attributes' => [
        'title' => $this->t('Cancel this action'),
        'class' => [
          'button--danger',
          'btn-link',
        ],
      ],
      '#submit' => ['::cancel'],
      '#limit_validation_errors' => [['nid']],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function cancel(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $nid = $values['nid'];
    $form_state->setRedirect('simply_signups.nodes.fields', array('node' => $values['nid']));
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $nid = $values['nid'];
    $id = $values['templates'];
    $requestTime = \Drupal::time()->getCurrentTime();
    $db = \Drupal::database();
    $query = $db->select('simply_signups_templates_fields', 'p');
    $query->fields('p');
    $query->condition('tid', $id, '=');
    $results = $query->execute()->fetchAll();
    $item = array();
    foreach ($results as $row) {
      $item['nid'] = $nid;
      $item['name'] = $row->name;
      $item['field'] = $row->field;
      $item['weight'] = $row->weight;
      $item['created'] = $requestTime;
      $item['updated'] = $requestTime;
      $db->insert('simply_signups_fields')
        ->fields($item)
        ->execute();
    }
    $form_state->setRedirect('simply_signups.nodes.fields', array('node' => $nid));
    drupal_set_message($this->t('Successfully applied template.'));
  }

}