<?php

namespace Drupal\simply_signups\Form;

use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\signup\Utility\SignupUtility;
use Drupal\Component\Datetime\TimeInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Implements a signup form.
 */
class SimplySignupsNodesFieldsAddForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'simply_signups_nodes_fields_add_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $path = \Drupal::service('path.current')->getPath();
    $currentPath = ltrim($path, '/');
    $arg = explode('/', $currentPath);
    $nid = $arg[1];
    $node = Node::load($nid);
    $isValidNode = (isset($node)) ? TRUE : FALSE;
    if (!$isValidNode) {
      throw new NotFoundHttpException();
    }
    $id = $node->id();
    $form['#attached']['library'][] = 'simply_signups/styles';
    $form['#attributes'] = [
      'class' => ['simply-signups-nodes-fields-add-form', 'simply-signups-form'],
    ];
    $form['nid']= [
      '#type' => 'hidden',
      '#value' => $id,
    ];
    $form['field_type'] = [
      '#type' => 'select',
      '#title' => t('Field Type'),
      '#options' => [
        'checkboxes' => t('Checkboxes'),
        'checkbox' => t('Checkbox'),
        'email' => t('Email'),
        'hidden' => t('Hidden'),
        'number' => t('Number'),
        'radio' => t('Radio'),
        'radios' => t('Radios'),
        'select' => t('Select'),
        'tel' => t('Telephone'),
        'textarea' => t('Textarea'),
        'textfield' => t('Textfield'),
      ],
      '#description' => t('Select the type of field that you wish to add.'),
    ];
    $form['actions'] = [
      '#type' => 'actions',
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => t('Select field type'),
      '#attributes' => [
        'class' => [
          'button--primary',
          'btn-primary',
        ],
      ],
    ];
    $form['actions']['cancel'] = [
      '#type' => 'submit',
      '#value'  => 'Cancel',
      '#attributes' => [
        'title' => $this->t('Return to template fields'),
        'class' => [
          'button--danger',
          'btn-link',
        ],
      ],
      '#submit' => ['::cancel'],
      '#limit_validation_errors' => [['nid']],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function cancel(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $nid = $values['nid'];
    $form_state->setRedirect('simply_signups.nodes.fields', array('node' => $nid));
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $nid = $values['nid'];
    $form_state->setRedirect('simply_signups.nodes.fields.'.$values['field_type'], array('node' => $nid));
  }
}