<?php

namespace Drupal\simply_signups\Form;

use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\Core\Path;
use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Render\Element\Token;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use CommerceGuys\Addressing\AddressFormat\AddressField;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;


/**
 * Implements a signup form.
 */
class SimplySignupsNodesSettingsForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'simply_signups_nodes_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $path = \Drupal::service('path.current')->getPath();
    $currentPath = ltrim($path, '/');
    $arg = explode('/', $currentPath);
    $nid = $arg[1];
    $node = Node::load($nid);
    $requestTime = \Drupal::time()->getCurrentTime();
    $isValidNode = (isset($node)) ? TRUE : FALSE;
    if (!$isValidNode) {
      throw new NotFoundHttpException();
    }
    $nid = $node->id();
    $title = $node->getTitle();
    $db = \Drupal::database();
    $query = $db->select('simply_signups_settings', 'p');
    $query->fields('p');
    $query->condition('nid', $nid, '=');
    $count = $query->countQuery()->execute()->fetchField();
    if ($count > 0) {
      $results = $query->execute()->fetchAll();
      foreach ($results as $row) {
        $nid = $row->nid;
        $startDate = DrupalDateTime::createFromTimestamp($row->start_date);
        $endDate = DrupalDateTime::createFromTimestamp($row->end_date);
        $eventStartDate = DrupalDateTime::createFromTimestamp($row->event_start_date);
        $eventEndDate = DrupalDateTime::createFromTimestamp($row->event_end_date);
        $eventName = $row->name;
        $eventLocation = $row->location;
        $addressLine1 = $row->address_line1;
        $addressLine2 = $row->address_line2;
        $administrativeArea = $row->administrative_area;
        $locality = $row->locality;
        $postalCode = $row->postal_code;
        $countryCode = $row->country_code;
        $altPath = $row->path;
        $maxSignups = $row->max_signups;
        $status = $row->status;
        $adminSendMail = $row->admin_send_mail;
        $adminMail = $row->admin_mail;
        $adminSubject = $row->admin_subject;
        $adminMessage = $row->admin_message;
        $adminFormat = $row->admin_format;
        $clientSendMail = $row->client_send_mail;
        $clientSubject = $row->client_subject;
        $clientMessage = $row->client_message;
        $clientFormat = $row->client_format;
        $updated = $row->updated;
      }
    }
    $form['#attached']['library'][] = 'simply_signups/styles';
    $form['#attributes'] = [
      'class' => ['simply-signups-nodes-settings-form', 'simply-signups-form'],
    ];
    $form['signup_node_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('<em>'. $title. ':</em> node settings'),
    ];
    $form['signup_node_fieldset']['nid'] = [
      '#type' => 'hidden',
      '#value' => ($node->id()) ? $node->id() : $nid,
    ];
    $form['signup_node_fieldset']['status'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled signups for this node'),
      '#default_value' => (isset($status)) ? $status : 0,
    ];
    $form['signup_node_fieldset']['start_date'] = [
      '#type' => 'datetime',
      '#title' => $this->t('Signups open'),
      '#required' => true,
      '#default_value' => (isset($startDate)) ? $startDate : '',
    ];
    $form['signup_node_fieldset']['end_date'] = [
      '#type' => 'datetime',
      '#title' => $this->t('Signups close'),
      '#required' => true,
      '#default_value' => (isset($endDate)) ? $endDate : '',
    ];
    $form['signup_node_fieldset']['path'] = [
      '#type' => 'url',
      '#title' => $this->t('Alternate confirmation URL'),
      '#maxlength' => 254,
      '#default_value' => (!empty($altPath)) ? $altPath : 'https://westerncentralny.aaa.com/events/travel/thank-you',
      '#description' => t('Enter an alternate confirmation url (absolute), if you want to redirect the client, when a new signup is submitted'),
    ];
    $form['signup_node_fieldset']['max_signups'] = [
      '#type' => 'number',
      '#title' => $this->t('Max attending'),
      '#precision' => 10,
      '#decimals' => 0,
      '#minimum' => 0,
      '#default_value' => (isset($maxSignups)) ? $maxSignups : 0,
      '#required' => true,
      '#description' => $this->t('Enter the number of allowed attendees, this will include the number of people that the attendee is bringing with them (0 is unlimited).'),
    ];
    $form['signup_event_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Event Information'),
    ];
    $form['signup_event_fieldset']['event_start_date'] = [
      '#type' => 'datetime',
      '#title' => $this->t('Event Start'),
      '#required' => true,
      '#default_value' => (isset($eventStartDate)) ? $eventStartDate : '',
    ];
    $form['signup_event_fieldset']['event_end_date'] = [
      '#type' => 'datetime',
      '#title' => $this->t('Event End'),
      '#required' => true,
      '#default_value' => (isset($eventEndDate)) ? $eventEndDate : '',
    ];
    $form['signup_event_fieldset']['event_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Event Name'),
      '#maxlength' => 254,
      '#default_value' => (isset($eventName)) ? $eventName : '',
    ];
    $form['signup_event_fieldset']['event_location'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Event Location'),
      '#maxlength' => 254,
      '#default_value' => (isset($eventLocation)) ? $eventLocation : '',
    ];
    $form['signup_event_fieldset']['event_address'] = [
      '#type' => 'address',
      '#default_value' => [
        'address_line1' => (isset($addressLine1)) ? $addressLine1 : '',
        'address_line2' => (isset($addressLine2)) ? $addressLine2 : '',
        'locality' => (isset($locality)) ? $locality : '',
        'postal_code' => (isset($postalCode)) ? $postalCode : '',
        'country_code' => (isset($countryCode)) ? $countryCode : 'US',
        'administrative_area' => (isset($administrativeArea)) ? $administrativeArea : 'NY',
      ],
      '#used_fields' => [
        AddressField::ADDRESS_LINE1,
        AddressField::ADDRESS_LINE2,
        AddressField::ADMINISTRATIVE_AREA,
        AddressField::LOCALITY,
        AddressField::POSTAL_CODE,
      ],
    ];
    $form['admin_mail_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Admin mail settings'),
    ];
    $form['admin_mail_fieldset']['admin_send_mail'] = [
      '#type' => 'checkbox',
      '#title' => t('Select if you wish to send an email to the address designated below, when a new signup is submitted'),
      '#default_value' => (isset($adminSendMail)) ? $adminSendMail : 0,
    ];
    $form['admin_mail_fieldset']['admin_mail'] = [
      '#type' => 'email',
      '#title' => $this->t('Signup admin email'),
      '#maxlength' => 254,
      '#default_value' => (isset($adminMail)) ? $adminMail : '',
      '#description' => t('Enter a valid email for who will recieve emails when a signup form is submitted.'),
      '#states' => [
        'required' => [
          ':input[name="admin_send_mail"]' => [
            'checked' => true,
          ],
        ],
      ],
    ];
    $form['admin_mail_fieldset']['admin_subject'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Signup admin subject'),
      '#maxlength' => 254,
      '#default_value' => (isset($adminSubject)) ? $adminSubject : 'RSVP for: [node:title]',
      '#states' => [
        'required' => [
          ':input[name="admin_send_mail"]' => [
            'checked' => true,
          ],
        ],
      ],
    ];
    $form['admin_mail_fieldset']['admin_message'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Signup admin message'),
      '#format' => (isset($adminFormat)) ? $adminFormat : NULL,
      '#default_value' => (isset($adminMessage)) ? $adminMessage : '<p>A RSVP has been submitted for an event below.</p><h3>[node:title]</h3><p>[node:url]<br /></p>',
      '#description' => t('Please make sure that all urls are absolute.'),
      '#states' => [
        'required' => [
          ':input[name="admin_send_mail"]' => [
            'checked' => true,
          ],
        ],
      ],
    ];
    $form['admin_mail_fieldset']['token_help_admin'] = [
      '#theme' => 'token_tree_link',
      '#token_types' => ['node'],
    ];
    $form['client_mail_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Client mail settings'),
    ];
    $form['client_mail_fieldset']['client_send_mail'] = [
      '#type' => 'checkbox',
      '#title' => t('Select if you wish to send an email to the client, when a new signup is submitted'),
      '#default_value' => (isset($clientSendMail)) ? $clientSendMail : 0,
    ];
    $form['client_mail_fieldset']['client_subject'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Signup client subject'),
      '#maxlength' => 254,
      '#default_value' => (isset($clientSubject)) ? $clientSubject : 'RSVP Confirmation: [node:title]',
      '#states' => [
        'required' => [
          ':input[name="client_send_mail"]' => [
            'checked' => true,
          ],
        ],
      ],
    ];
    $form['client_mail_fieldset']['client_message'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Signup client message'),
      '#format' => (isset($clientFormat)) ? $clientFormat : NULL,
      '#default_value' => (isset($clientMessage)) ? $clientMessage : "<h3>Thank you for your RSVP!</h3><p>We're happy to see that you're interested in coming to our event, [node:title]!</p><p>You can find event details at [node:url]</p><p>If you are no longer able to attend the event, that's fine! We kindly ask that you let us know by contacting us at: [site:mail].</p><p>Thank you! We look forward to seeing you soon.</p><p>Sincerely,<br />[site:name]</p>",
      '#description' => t('Please make sure that all urls are absolute.'),
      '#states' => [
        'required' => [
          ':input[name="client_send_mail"]' => [
            'checked' => true,
          ],
        ],
      ],
    ];
    $form['client_mail_fieldset']['token_help_client'] = [
      '#theme' => 'token_tree_link',
      '#token_types' => ['node'],
    ];
    $form['actions'] = [
      '#type' => 'actions',
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Configure signup'),
      '#attributes' => [
        'class' => [
          'button--primary',
          'btn-primary',
        ],
      ],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    if (!empty($values['path'])) {
      $isValidAbsolute = UrlHelper::isValid($values['path'], $absolute = TRUE);
      if (($isValidAbsolute == FALSE)) {
        $form_state->setErrorByName('path', t('You MUST enter a VALID alternate confirmation url'));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $requestTime = \Drupal::time()->getCurrentTime();
    $db = \Drupal::database();
    $query = $db->select('simply_signups_settings', 'p');
    $query->fields('p');
    $query->condition('nid', $values['nid'], '=');
    $count = $query->countQuery()->execute()->fetchField();
    $startDatetime = $values['start_date'];
    $endDatetime = $values['end_date'];

    $startTimestamp = strtotime($startDatetime->format('Y-m-d h:i:s A'));
    $endTimestamp = strtotime($endDatetime->format('Y-m-d h:i:s A'));

    $eventStartDateTime = $values['event_start_date'];
    $eventEndDateTime = $values['event_end_date'];

    $eventStartTimestamp = strtotime($eventStartDateTime->format('Y-m-d h:i:s A'));
    $eventEndTimestamp = strtotime($eventEndDateTime->format('Y-m-d h:i:s A'));

    $eventName = $values['event_name'];
    $eventLocation = $values['event_location'];
    $addressLine1 = $values['event_address']['address_line1'];
    $addressLine2 = $values['event_address']['address_line2'];
    $administrativeArea = $values['event_address']['administrative_area'];
    $locality = $values['event_address']['locality'];
    $postal_code = $values['event_address']['postal_code'];
    $country_code = $values['event_address']['country_code'];
    $altPath = (!empty($values['path'])) ? ltrim($values['path'], '/') : '';
    $adminMail = $values['admin_mail'];
    $adminSendMail = (isset($values['admin_send_mail'])) ? $values['admin_send_mail'] : 0;
    $adminSubject = (isset($values['admin_subject'])) ? $values['admin_subject'] : '';
    $adminMessage = (isset($values['admin_message']['value'])) ? $values['admin_message']['value'] : '';
    $adminFormat = (isset($values['admin_message']['format'])) ? $values['admin_message']['format'] : '';
    $clientSendMail = (isset($values['client_send_mail'])) ? $values['client_send_mail'] : 0;
    $clientSubject = (isset($values['client_subject'])) ? $values['client_subject'] : '';
    $clientMessage = (isset($values['client_message']['value'])) ? $values['client_message']['value'] : '';
    $clientFormat = (isset($values['client_message']['format'])) ? $values['client_message']['format'] : '';
    if ($count > 0) {
      $query = $db->update('simply_signups_settings');
      $query->fields([
        'start_date' => $startTimestamp,
        'end_date' => $endTimestamp,
        'event_start_date' => $eventStartTimestamp,
        'event_end_date' => $eventEndTimestamp,
        'name' => $eventName,
        'location' => $eventLocation,
        'address_line1'  => $addressLine1,
        'address_line2'  => $addressLine2,
        'locality'  => $locality,
        'administrative_area'  => $administrativeArea,
        'postal_code'  => $postal_code,
        'country_code'  => $country_code,
        'path' => $altPath,
        'max_signups' => $values['max_signups'],
        'status' => $values['status'],
        'admin_mail' => $adminMail,
        'admin_send_mail' => $adminSendMail,
        'admin_subject' => $adminSubject,
        'admin_message' => $adminMessage,
        'admin_format' => $adminFormat,
        'client_send_mail' => $clientSendMail,
        'client_subject' => $clientSubject,
        'client_message' => $clientMessage,
        'client_format' => $clientFormat,
        'updated' => $requestTime,
      ]);
      $query->condition('nid', $values['nid'], '=');
      $query->execute();
    }
    if ($count == 0) {
      $query = $db->insert('simply_signups_settings');
      $query->fields([
        'nid' => $values['nid'],
        'start_date' => $startTimestamp,
        'end_date' => $endTimestamp,
        'event_start_date' => $eventStartTimestamp,
        'event_end_date' => $eventEndTimestamp,
        'name' => $eventName,
        'location' => $eventLocation,
        'address_line1'  => $addressLine1,
        'address_line2'  => $addressLine2,
        'locality'  => $locality,
        'administrative_area'  => $administrativeArea,
        'postal_code'  => $postal_code,
        'country_code'  => $country_code,
        'path' => $altPath,
        'max_signups' => $values['max_signups'],
        'status' => $values['status'],
        'admin_mail' => $adminMail,
        'admin_send_mail' => $adminSendMail,
        'admin_subject' => $adminSubject,
        'admin_message' => $adminMessage,
        'admin_format' => $adminFormat,
        'client_send_mail' => $clientSendMail,
        'client_subject' => $clientSubject,
        'client_message' => $clientMessage,
        'client_format' => $clientFormat,
        'updated' => $requestTime,
        'created' => $requestTime,
      ]);
      $query->execute();
    }
    drupal_set_message($this->t('Your node\'s signup settings have been saved successfully.'));
  }
}