<?php

namespace Drupal\simply_signups\Form;

use Drupal\Core\Url;
use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Implements a signup form.
 */
class SimplySignupsNodesSingleForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'simply_signups_nodes_single_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $path = \Drupal::service('path.current')->getPath();
    $currentPath = ltrim($path, '/');
    $arg = explode('/', $currentPath);
    $nid = $arg[1];
    $sid = $arg[3];
    $node = Node::load($nid);
    $id = $node->id();
    $db = \Drupal::database();
    $query = $db->select('simply_signups_data', 'p');
    $query->fields('p');
    $query->condition('id', $sid, '=');
    $signupCount = $query->countQuery()->execute()->fetchField();
    if ($signupCount == 0) {
      throw new NotFoundHttpException();
    }
    $results = $query->execute()->fetchAll();
    foreach ($results as $row) {
      $fields = unserialize($row->fields);
      $statusData = $row->status;
      $status = ($row->status = 1) ? '<div><strong>Status:</strong> - </div>' : '<div><strong>Status:</strong> Checked In</div>';
    }
    $data = '';
    foreach ($fields as $field) {
      $data .= '<div><strong>'. $field['title']. ':</strong> '. $field['value']. '</div>';
    }
    $data .= $status;
    $form['#attached']['library'][] = 'simply_signups/styles';
    $form['#attributes'] = [
      'class' => ['simply-signups-single-form', 'simply-signups-form'],
    ];
    $form['nid'] = [
      '#type' => 'hidden',
      '#value' => $nid,
    ];
    $form['sid'] = [
      '#type' => 'hidden',
      '#value' => $sid,
    ];
    $form['status'] = [
      '#type' => 'hidden',
      '#value' => ($statusData == 1) ? 1 : 0,
    ];
    $form['signup_entry'] = [
      '#markup' => t('<div>'. $data. '</div>'),
    ];
    $form['actions'] = [
      '#type' => 'actions',
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => ($statusData == 1) ? $this->t('Check Out') : $this->t('Check In'),
      '#attributes' => [
        'class' => ($statusData == 1) ? ['btn-primary'] : ['btn-success'],
      ],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $requestTime = \Drupal::time()->getCurrentTime();
    $updatedStatus = ($values['status'] == 1) ? 0 : 1;
    $row = [
      'status' => $updatedStatus,
      'updated' => $requestTime,
    ];
    $db = \Drupal::database();
    $db->update('simply_signups_data')
      ->fields($row)
      ->condition('id', $values['sid'], '=')
      ->execute();
    $message = ($updatedStatus == 1) ? 'Successfully <strong>checked in</strong> current signup entry.' : 'Successfully <strong>checked out</strong> current signup entry.'; 
    $form_state->setRedirect('simply_signups.nodes', array('node' => $values['nid']));
    drupal_set_message($this->t($message));
  }
}