<?php

namespace Drupal\simply_signups\Utility;

use Drupal\Component\Utility\Html;

class SimplySignupsUtility {
  public static function sanitize_options($entityoptions) {
    $lines = explode(PHP_EOL, $entityoptions);
    $options = array();
    foreach($lines as $line) {
      $containsPipe = preg_match("/\|/", $line);
      if ($containsPipe == 1) {
        $line = explode('|', $line);
        $line[0] = Html::escape($line[0]);
        $line[1] = Html::escape($line[1]);
        $options[$line[0]] = $line[1];
      }
      else {
        $line = Html::escape($line);
        $line = rtrim($line);
        $options[$line] = $line;
      }
    }
    return $options;
  }

  public static function sanitize_options_select($entityoptions, $required) {
    $lines = explode(PHP_EOL, $entityoptions);
    $options = array();
    if ($required == 1) {
      $options[''] = '- Select -';
    }
    else {
      $options[''] = '- None -';
    }
    foreach($lines as $line) {
      $containsPipe = preg_match("/\|/", $line);
      if ($containsPipe == 1) {
        $line = explode('|', $line);
        $line[0] = Html::escape($line[0]);
        $line[1] = Html::escape($line[1]);
        $options[$line[0]] = $line[1];
      }
      else {
        $line = Html::escape($line);
        $line = rtrim($line);
        $options[$line] = $line;
      }
    }
    return $options;
  }

  public static function sanitize_options_textarea($entityoptions) {
    $options = '';
    $entityoptions = array_filter($entityoptions);
    foreach ($entityoptions as $key => $value) {
      if (($value != '- Select -') AND ($value != '- None -')) {
        $options .= trim($key). "|". trim($value). "\n";
      }
    }
    return trim($options);
  }

  /*
   * Return a tally of the number of attending colun in the sign data table.
   *
   */
  public static function get_number_of_attending($node) {
    if (is_numeric($node)) {
      $db = \Drupal::database();
      $query = $db->select('simply_signups_data', 'p');
      $query->fields('p');
      $query->condition('nid', $node, '=');
      $count = $query->countQuery()->execute()->fetchField();
      if ($count > 0) {
        $results = $query->execute()->fetchAll();
        $attending = 0;
        foreach ($results as $row) {
          $attending = ($attending+$row->attending);
        }
        return $attending;
      }
    }
    return '0';
  }

  /*
   * Return the number of rows that are 'signed up' for $node.
   *
   */
  public static function get_number_of_signups($node) {
    if (is_numeric($node)) {
      $db = \Drupal::database();
      $query = $db->select('simply_signups_data', 'p');
      $query->fields('p');
      $query->condition('nid', $node, '=');
      $count = $query->countQuery()->execute()->fetchField();
      return $count;
    }
    return '0';
  }

  /*
   * Return the number of rows that are 'signed up' for $node
   * where status is 1 (checked-in).
   *
   */
  public static function get_number_of_checkins($node) {
    if (is_numeric($node)) {
      $db = \Drupal::database();
      $query = $db->select('simply_signups_data', 'p');
      $query->fields('p');
      $query->condition('nid', $node, '=');
      $query->condition('status', 1, '=');
      $count = $query->countQuery()->execute()->fetchField();
      return $count;
    }
    return FALSE;
  }

  /*
   * Return the number of attending for $node
   * where status is 1 (checked-in).
   *
   */
  public static function get_number_of_checkedins_attending($node) {
    if (is_numeric($node)) {
      $db = \Drupal::database();
      $query = $db->select('simply_signups_data', 'p');
      $query->fields('p');
      $query->condition('nid', $node, '=');
      $query->condition('status', 1, '=');
      $count = $query->countQuery()->execute()->fetchField();
      if ($count > 0) {
        $results = $query->execute()->fetchAll();
        $tally = 0;
        foreach ($results as $row) {
          $tally = ($tally + $row->attending);
        }
        return $tally;
      }
    }
    return 0;
  }

  /*
   * Return the number of fields that are assigned to the 
   * signup form attached to a particular node id.
   *
   */
  public static function get_number_of_fields($node) {
    if (is_numeric($node)) {
      $db = \Drupal::database();
      $query = $db->select('simply_signups_fields', 'p');
      $query->fields('p');
      $query->condition('nid', $node, '=');
      $count = $query->countQuery()->execute()->fetchField();
      return $count;
    }
    return FALSE;
  }

  public static function get_start_date($node) {
    if (is_numeric($node)) {
      $db = \Drupal::database();
      $query = $db->select('simply_signups_settings', 'p');
      $query->fields('p');
      $query->condition('nid', $node, '=');
      $count = $query->countQuery()->execute()->fetchField();
      if ($count > 0) {
        $results = $query->execute()->fetchAll();
        foreach ($results as $row) {
          $startDate = $row->start_date;
        }
        return $startDate;
      }
    }
    return FALSE;
  }

  public static function get_end_date($node) {
    if (is_numeric($node)) {
      $db = \Drupal::database();
      $query = $db->select('simply_signups_settings', 'p');
      $query->fields('p');
      $query->condition('nid', $node, '=');
      $count = $query->countQuery()->execute()->fetchField();
      if ($count > 0) {
        $results = $query->execute()->fetchAll();
        foreach ($results as $row) {
          $endDate = $row->end_date;
        }
        return $endDate;
      }
    }
    return FALSE;
  }

  public static function get_max_attending($node) {
    if (is_numeric($node)) {
      $db = \Drupal::database();
      $query = $db->select('simply_signups_settings', 'p');
      $query->fields('p');
      $query->condition('nid', $node, '=');
      $count = $query->countQuery()->execute()->fetchField();
      if ($count > 0) {
        $results = $query->execute()->fetchAll();
        foreach ($results as $row) {
          $maxSignups = $row->max_signups;
        }
        return $maxSignups;
      }
    }
    return FALSE;
  }

  public static function format_telephone($telephone, $format) {
    $pattern = "/[^0-9]/";
    $number = preg_replace($pattern,"", $telephone);
    $formattedTelephone = FALSE;
    if (is_numeric($number)) {
      if (strlen((string)$number) == 10) {
        $telephone_piece = array();
        $telephone_piece[0] = substr($number, 0, 3);
        $telephone_piece[1] = substr($number, 3, 3);
        $telephone_piece[2] = substr($number, 6, 10);
        if ($format == 1) {
          $formattedTelephone = '('. $telephone_piece[0]. ') '. $telephone_piece[1]. ' '. $telephone_piece[2];
        }
        if ($format == 2) {
          $formattedTelephone = '('. $telephone_piece[0]. ') '. $telephone_piece[1]. '-'. $telephone_piece[2];
        }
        else {
          $formattedTelephone = $telephone_piece[0]. '-'. $telephone_piece[1]. '-'. $telephone_piece[2];
        }
      }
    }
    return $formattedTelephone;
  }
}
